<?php

namespace Drupal\speakeasy\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure SpeakEasy settings for this site.
 */
class SpeakEasySettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'speakeasy_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['speakeasy.settings'];
  }

  /**
   * Detect the current browser based on the User-Agent header.
   */
  private function getCurrentBrowser() {
    $ua = strtolower($_SERVER['HTTP_USER_AGENT'] ?? '');
    if (strpos($ua, 'chrome') !== FALSE && strpos($ua, 'edg') === FALSE) {
      return 'chrome';
    }
    elseif (strpos($ua, 'edg') !== FALSE) {
      return 'edge';
    }
    elseif (strpos($ua, 'firefox') !== FALSE) {
      return 'firefox';
    }
    elseif (strpos($ua, 'safari') !== FALSE && strpos($ua, 'chrome') === FALSE) {
      return 'safari';
    }
    return 'other';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('speakeasy.settings');
    $default_voices = $config->get('default_voices_by_browser') ?: [];
    $current_browser = $this->getCurrentBrowser();

    \Drupal::logger('speakeasy')->debug('Loaded configuration on form build: @config, Current browser: @browser', [
      '@config' => print_r($default_voices, TRUE),
      '@browser' => $current_browser,
    ]);

    $form['#attached']['library'][] = 'speakeasy/speakeasy.settings';

    $form['default_voices_by_browser'] = [
      '#type' => 'details',
      '#title' => $this->t('Default Voices by Browser'),
      '#description' => $this->t('Specify default voices for each browser. Leave blank for browser default. Autocomplete uses available voices for the current browser and fallbacks for others.'),
      '#open' => TRUE,
    ];

    $browsers = [
      'chrome' => $this->t('Google Chrome'),
      'edge' => $this->t('Microsoft Edge'),
      'firefox' => $this->t('Mozilla Firefox'),
      'safari' => $this->t('Apple Safari'),
      'other' => $this->t('Other Browsers'),
    ];

    foreach ($browsers as $key => $label) {
      $form['default_voices_by_browser'][$key] = [
        '#type' => 'textfield',
        '#title' => $label,
        '#default_value' => $default_voices[$key] ?? '',
        '#autocomplete_route_name' => 'speakeasy.voice_autocomplete',
        '#autocomplete_route_parameters' => ['browser' => $key],
        '#description' => $this->t('Enter the voice name for @browser. Autocomplete uses @source.', [
          '@browser' => $label,
          '@source' => ($key === $current_browser) ? 'available voices from your browser' : 'a fallback list',
        ]),
      ];
    }

    $form['default_rate'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Rate'),
      '#description' => $this->t('Default speech rate (0.1 to 10).'),
      '#default_value' => $config->get('default_rate') ?: 1,
      '#step' => 0.1,
      '#min' => 0.1,
      '#max' => 10,
    ];

    $form['default_pitch'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Pitch'),
      '#description' => $this->t('Default speech pitch (0 to 2).'),
      '#default_value' => $config->get('default_pitch') ?: 1,
      '#step' => 0.1,
      '#min' => 0,
      '#max' => 2,
    ];

    // Hidden field to store real-time voices from JavaScript.
    $form['current_voices'] = [
      '#type' => 'hidden',
      '#default_value' => '',
      '#attributes' => ['id' => 'speakeasy-current-voices'],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $voices_by_browser = [];
    $browsers = ['chrome', 'edge', 'firefox', 'safari', 'other'];

    $all_values = $form_state->getValues();
    \Drupal::logger('speakeasy')->debug('All submitted form state values: @values', [
      '@values' => print_r($all_values, TRUE),
    ]);

    foreach ($browsers as $browser) {
      $voice = $form_state->getValue($browser) ?? '';
      $trimmedVoice = trim($voice);
      if (!empty($trimmedVoice)) {
        $voices_by_browser[$browser] = $trimmedVoice;
      }
    }

    \Drupal::logger('speakeasy')->debug('Processed voices to save: @voices', [
      '@voices' => print_r($voices_by_browser, TRUE),
    ]);

    $config = $this->config('speakeasy.settings');
    $config
      ->set('default_voices_by_browser', $voices_by_browser)
      ->set('default_rate', $form_state->getValue('default_rate'))
      ->set('default_pitch', $form_state->getValue('default_pitch'))
      ->save();

    $saved_config = $config->get();
    \Drupal::logger('speakeasy')->debug('Saved configuration: @config', [
      '@config' => print_r($saved_config, TRUE),
    ]);

    $this->messenger()->addStatus($this->t('SpeakEasy settings have been saved.'));

    parent::submitForm($form, $form_state);
  }

}